package rocketBugsRender;
/*
	NR ... methods from "Numerical Recipes in C"
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.math.*;

public class NR {

	public NR () {}
	
	
	public static double [] avevar(double [] data, int ctInt) {
		//Given array data[1..n], returns its mean as ave and its variance as var.
		double ct = (double)ctInt;	// use this one for division
		double s;
		double ave = 0;
		double ep = 0;
		double var = 0;
		for (int j=0;j<ctInt;j++) { ave += data[j]; }
		ave /= ct;
		for (int j=0;j<ctInt;j++) {
			s=data[j]-ave;
			ep += s;
			var += s*s;
		}
		var=(var-ep*ep/ct)/(ct-1); // Corrected two-pass formula (14.1.8).
		return new double [] {ave,var};
	}
	
	
	public static double gammln(double inX) {
		//Returns the value ln[F(xx)] for xx > 0.
		//Internal arithmetic will be done in double precision, a nicety that you can omit if 5-figure accuracy is good enough.
		double x,y,tmp,ser;
		double [] cof = new double [] {76.18009172947146,-86.50532032941677, 24.01409824083091,-1.231739572450155, 0.1208650973866179e-2,-0.5395239384953e-5};
		y=x=inX;
		tmp=x+5.5;
		tmp -= (x+0.5)*Math.log(tmp);
		ser=1.000000000190015;
		for (int j=0;j<=5;j++) { ser += cof[j]/y++; }
		return -tmp+Math.log(2.5066282746310005*ser/x);
	}

	public static double gser(double a, double x) {
		//Returns the incomplete gamma function P(a, x) evaluated by its series representation as gamser.
		//6.2 Incomplete Gamma Function 219
		int ITMAX = 100;
		double EPS = 3.0e-7;
		double sum,del,ap;
        double gln=gammln(a);
		if (x <= 0.0) {
			if (x < 0.0) { talkln("x less than 0 in routine gser"); }
			return 0;
		} else {
			ap=a;
			del=sum=1.0/a;
			for (int i=0;i<ITMAX;i++) {
				ap++;
				del *= x/ap;
				sum += del;
				if (Math.abs(del) < Math.abs(sum)*EPS) {
					return sum*Math.exp(-x+a*Math.log(x)-gln);
				}
			}
			talkln ("a too large, ITMAX too small in routine gser");
			return 0;
		}
	}
	
	
	public static double gcf(double a, double x) {
		// Returns the incomplete gamma function Q(a, x) evaluated by its continued fraction representation
		int ITMAX = 100;	// Maximum allowed number of iterations.
		double EPS = 3.0e-7; // Relative accuracy.
		double FPMIN = 1.0e-30;  // Number near the smallest representable floating-point number.
		double an,b,c,d,del,h;
		double gln=gammln(a);
		b=x+1.0-a; 	//Set up for evaluating continued fraction by modified Lentzs method (5.2) with b0 = 0.
		c=1.0/FPMIN;
		d=1.0/b;
		h=d;
		int i;
		for (i=1;i<=ITMAX;i++) { //Iterate to convergence.
			an = -i*(i-a);
			b += 2.0;
			d=an*d+b;
			if (Math.abs(d) < FPMIN) { d=FPMIN; }
			c=b+an/c;
			if (Math.abs(c) < FPMIN) { c=FPMIN; }
			d=1.0/d;
			del=d*c;
			h *= del;
			if (Math.abs(del-1.0) < EPS) { return 0; }
		}
		if (i > ITMAX) { talkln("a too large, ITMAX too small in gcf"); }
		return Math.exp(-x+a*Math.log(x)-gln)*h; 	//Put factors in front.
	}
	
	public static double gammp(double a, double x) {
		//Returns the incomplete gamma function P(a, x).
		double gamser,gammcf,gln;
		if ((x < 0.0) || (a <= 0.0)) { talkln("Invalid arguments in routine gammq"); }
		if (x < (a+1.0)) { 	// Use the series representation.
			gamser = gser(a,x);
			return gamser;
		} else { // Use the continued fraction representation
			gammcf = gcf(a,x);
			return 1.0-gammcf; //and take its complement.
		}
	}
	
	public static double gammq(double a, double x) {
		//Returns the incomplete gamma function Q(a, x)  1 - P(a, x).
		double gamser,gammcf,gln;
		if ((x < 0.0) || (a <= 0.0)) { talkln("Invalid arguments in routine gammq"); }
		if (x < (a+1.0)) { 	//Use the series representation
			gamser = gser(a,x);
			return 1.0-gamser; // and take its complement.
		} else { 	//Use the continued fraction representation.
			gammcf = gcf(a,x);
			return gammcf;
		}
	}
	
	
/*	public static double brent (double ax, double bx, double cx, Object F, double tol, double xmin) {
		int ITMAX = 100;
		double CGOLD = 0.3819660;
		double ZEPS = 1.0e-10;
		int iter;
		double a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
		double e=0.0; //This will be the distance moved on the step before last.
*/	
//#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);
		/*
		Given a function f, and given a bracketing triplet of abscissas ax, bx, cx (such that bx is
		between ax and cx, and f(bx) is less than both f(ax) and f(cx)), this routine isolates
		the minimum to a fractional precision of about tol using Brents method. The abscissa of
		the minimum is returned as xmin, and the minimum function value is returned as brent, the
		returned function value.
		*/
	
/* All below must still be converted to JAVA
		a=(ax < cx ? ax : cx); a and b must be in ascending order,but input abscissas need not be. 
		b=(ax > cx ? ax : cx);
		x=w=v=bx; Initializations...
		fw=fv=fx=(*f)(x);
for (iter=1;iter<=ITMAX;iter++) { Main program loop.
xm=0.5*(a+b);
tol2=2.0*(tol1=tol*fabs(x)+ZEPS);
if (fabs(x-xm) <= (tol2-0.5*(b-a))) { Test for done here.
*xmin=x;
return fx;
}
if (fabs(e) > tol1) { Construct a trial parabolic fit.
r=(x-w)*(fx-fv);
q=(x-v)*(fx-fw);
p=(x-v)*q-(x-w)*r;
q=2.0*(q-r);
if (q > 0.0) p = -p;
q=fabs(q);
etemp=e;
e=d;
if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
d=CGOLD*(e=(x >= xm ? a-x : b-x));
The above conditions determine the acceptability of the parabolic fit. Here we
take the golden section step into the larger of the two segments.
else {
d=p/q; Take the parabolic step.
u=x+d;
if (u-a < tol2 || b-u < tol2)
d=SIGN(tol1,xm-x);
}
} else {
d=CGOLD*(e=(x >= xm ? a-x : b-x));
}
u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));
fu=(*f)(u);
This is the one function evaluation per iteration.
if (fu <= fx) { Now decide what to do with our function
evaluation. if (u >= x) a=x; else b=x;
SHFT(v,w,x,u) Housekeeping follows:
SHFT(fv,fw,fx,fu)
} else {
if (u < x) a=u; else b=u;
if (fu <= fw || w == x) {
v=w;
w=u;
fv=fw;
fw=fu;
} else if (fu <= fv || v == x || v == w) {
v=u;
fv=fu;
}
} Done with housekeeping. Back for
another iteration. }
nrerror("Too many iterations in brent");
*xmin=x; Never get here.
return fx; 
	} */
	
	private static void talk (String info) {
		//System.out.print(info);
	}
	
	private static void talkln (String info) {
		//System.out.println(info);
	}
	
}